@@ -66,7 +66,7 @@ class App:
         # pylint: disable=import-outside-toplevel
         from ansiblelint.yaml_utils import load_yamllint_config
 
-        self.yamllint_config = load_yamllint_config()
+        self.yamllint_config = load_yamllint_config(options.yamllint_file)
 
     def render_matches(self, matches: list[MatchError]) -> None:
         """Display given matches (if they are not fixed)."""
@@ -104,7 +104,11 @@ class App:
                     console.print(self.formatter.apply(match))
 
         # If run under GitHub Actions we also want to emit output recognized by it.
-        if os.getenv("GITHUB_ACTIONS") == "true" and os.getenv("GITHUB_WORKFLOW"):
+        if (
+            os.getenv("GITHUB_ACTIONS") == "true"
+            and os.getenv("GITHUB_WORKFLOW")
+            and os.getenv("GITHUB_ACTIONS_TEST", "false") == "false"
+        ):
             _logger.info(
                 "GitHub Actions environment detected, adding annotations output...",
             )
@@ -295,7 +299,8 @@ class App:
         summary.sort()
 
         if changed_files_count:
-            console_stderr.print(f"Modified {changed_files_count} files.")
+            file_word = "file" if changed_files_count == 1 else "files"
+            console_stderr.print(f"Modified {changed_files_count} {file_word}.")
 
         # determine which profile passed
         summary.passed_profile = ""
@@ -418,4 +423,7 @@ def get_app(*, offline: bool | None = None, cached: bool = False) -> App:
         role_name_check=role_name_check,
     )
 
+    # Enable plugin loader now that collections are installed
+    app.runtime.enable_plugin_loader()
+
     return app
